-- name: Minimap+ v1.1
-- description: The minimap from MarioHunt, now as a standalone mod.\n\nMod by EmilyEmmi\nCharacter select support by Squishy\nSpecial thanks to RoxasYTB for making the map images and Caec for the original Minimap mod.

-- this reduces lag apparently
local djui_hud_set_font, djui_hud_set_resolution, is_game_paused, get_current_save_file_num, save_file_get_star_flags, mod_storage_save, mod_storage_load =
    djui_hud_set_font, djui_hud_set_resolution, is_game_paused, get_current_save_file_num, save_file_get_star_flags,
    mod_storage_save, mod_storage_load
local PST = gPlayerSyncTable
local NetP = gNetworkPlayers
local MST = gMarioStates
local m0, sMario0, np0 = MST[0], PST[0], NetP[0]

-- player settings
showMiniMap = true
if mod_storage_load("showMiniMap") == "false" then showMiniMap = false end

function on_hud_render()
  -- render to N64 screen space, with the NORMAL font
  djui_hud_set_resolution(RESOLUTION_N64)
  djui_hud_set_font(FONT_NORMAL)

  local actSelect = (np0.currAreaIndex == 2 and obj_get_first_with_behavior_id(id_bhvActSelector) ~= nil)
  if sMario0.inActSelect ~= actSelect then
    sMario0.inActSelect = actSelect
  end

  if not (is_game_paused() or actSelect) and showMiniMap then
    render_minimap() -- first create minimap

    -- work with boxes
    local o = obj_get_first_with_behavior_id(id_bhvExclamationBox)
    while o do
      if exclamation_box_valid[o.oBehParams2ndByte] and o.oAction ~= 6 and o.header.gfx.node.flags & GRAPH_RENDER_ACTIVE ~= 0 then
        local star = (o.oBehParams >> 24) + 1
        if o.oBehParams2ndByte ~= 8 then
          star = o.oBehParams2ndByte - 8
        end
        if star == 0 then print("ERROR!") end
        local file = get_current_save_file_num() - 1
        local course_star_flags = save_file_get_star_flags(file, np0.currCourseNum - 1)
        if course_star_flags & (1 << (star - 1)) == 0 then
          render_on_minimap(o, box_minimap[star], true, "box")
        end
      end
      o = obj_get_next_with_same_behavior_id(o)
    end

    -- red coins
    o = obj_get_nearest_object_with_behavior_id(m0.marioObj, id_bhvRedCoin)
    if o then
      render_on_minimap(o, ex_minimap[1], true, "coin")
    end
    -- secrets
    o = obj_get_nearest_object_with_behavior_id(m0.marioObj, id_bhvHiddenStarTrigger)
    if o then
      render_on_minimap(o, ex_minimap[2], true, "secret")
    end

    -- player radar
    for i = 1, (MAX_PLAYERS - 1) do
      local m = MST[i]
      if is_player_active(m) ~= 0 then
        render_on_minimap(m, icon_minimap[i], false)
      end
    end
    if not (is_game_paused() or actSelect) and showMiniMap then
      render_player_minimap() -- myself
    end

    do_star_stuff()
  end
end
hook_event(HOOK_ON_HUD_RENDER, on_hud_render)

-- star minimap
function do_star_stuff()
  local didMinimap = {}
  local o = obj_get_first(OBJ_LIST_LEVEL)
  while o do
    if o.oInteractType == INTERACT_STAR_OR_KEY and (o.header.gfx.node.flags & GRAPH_RENDER_ACTIVE ~= 0 or (o.oRoom ~= -1 and current_mario_room_check(o.oRoom) == 0)) then
      local star = (o.oBehParams >> 24) + 1
      if (star > 0 and star < 8) and not didMinimap[star] then
        local file = get_current_save_file_num() - 1
        local course_star_flags = save_file_get_star_flags(file, gNetworkPlayers[0].currCourseNum - 1)

        if course_star_flags & (1 << (star - 1)) == 0 then
          if not star_minimap[star] then
            star_minimap[star] = { tex = gTextures.star, prevX = 0, prevY = 0 }
          end
          render_on_minimap(o, star_minimap[star], true)
          didMinimap[star] = 1
        end
      end
    end
    o = obj_get_next(o)
  end
end

-- converts hex string to RGB values
function convert_color(text)
  if text:sub(2, 2) ~= "#" then
    return nil
  end
  text = text:sub(3, -2)
  local rstring, gstring, bstring = "", "", ""
  if text:len() ~= 3 and text:len() ~= 6 then return 255, 255, 255, 255 end
  if text:len() == 6 then
    rstring = text:sub(1, 2) or "ff"
    gstring = text:sub(3, 4) or "ff"
    bstring = text:sub(5, 6) or "ff"
  else
    rstring = text:sub(1, 1) .. text:sub(1, 1)
    gstring = text:sub(2, 2) .. text:sub(2, 2)
    bstring = text:sub(3, 3) .. text:sub(3, 3)
  end
  local r = tonumber("0x" .. rstring) or 255
  local g = tonumber("0x" .. gstring) or 255
  local b = tonumber("0x" .. bstring) or 255
  return r, g, b, 255 -- alpha is no longer writeable
end

-- thank you sunk
exclamation_box_valid = {
  [8] = true,
  [10] = true,
  [11] = true,
  [12] = true,
  [13] = true,
  [14] = true
}

function minimap_command()
  showMiniMap = not showMiniMap
  mod_storage_save("showMiniMap", tostring(showMiniMap))
  return true
end

hook_chat_command("minimap", "- Toggle minimap", minimap_command)
