-- based on arena; this handles the minimap

-- this reduces lag apparently
local djui_hud_set_color, get_texture_info, djui_hud_render_texture_interpolated, djui_hud_set_resolution, network_get_player_text_color_string, obj_has_model_extended =
    djui_hud_set_color, get_texture_info, djui_hud_render_texture_interpolated, djui_hud_set_resolution,
    network_get_player_text_color_string, obj_has_model_extended

-- arg 1 is the texure name, arg 2 is the level size (for most levels, this is 8192, which is the default)
local minimap_data = {       -- supported rom hack data (not much here)
  [91] = { "bob-map" },      -- BOB area 1
  [241] = { "wf-map" },      -- WF area 1
  [161] = { "cg-map" },      -- CG area 1
  [61] = { "ic-map" },       -- IC area 1
  [62] = { "ica-map" },      -- IC area 2
  [63] = { "icb-map" },      -- IC area 3
  [171] = { "fb-map" },      -- B1 area 1
  [301] = { "fbb-map" },     -- B1 area 2
  [191] = { "bfs-map" },     -- B2 area 1
  [331] = { "bfsb-map" },    -- B2 area 2
  [51] = { "ccm-map" },      -- CCM area 1
  [52] = { "ccms-map" },     -- CCM area 1
  [121] = { "jrb-map" },     -- JRB area 1
  [122] = { "jrbs-map" },    -- JRB area 2
  [271] = { "pss-map" },     -- PSS area 1
  [41] = { "bbh-map" },      -- BBH area 1
  [261] = { "cc-map" },      -- BBH area 1
  [71] = { "hmc-map" },      -- HMC area 1
  [221] = { "lll-map" },     -- LLL area 1
  [222] = { "lllv-map" },    -- LLL area 2
  [81] = { "ssl-map" },      -- SSL area 1
  [82] = { "sslp-map" },     -- SSL area 2
  [83] = { "sslie-map" },    -- SSL area 2
  [231] = { "ddd-map" },     -- SSL area 2
  [232] = { "ddds-map" },    -- SSL area 2
  [101] = { "sl-map" },      -- SL area 1
  [102] = { "sli-map" },     -- SL area 2
  [111] = { "wdw-map" },     -- WDW area 1
  [112] = { "wdwu-map" },    -- WDW area 2
  [361] = { "ttm-map" },     -- TTM area 1
  -- [362] = { "ttms-map" },    -- TTM area 2 (removed due to inaccuracy)
  -- [363] = { "ttms-map" },    -- TTM area 3 (removed due to inaccuracy)
  -- [364] = { "ttms-map" },    -- TTM area 4 (removed due to inaccuracy)
  [131] = { "thih-map" },    -- THI area 1
  [132] = { "thit-map" },    -- THI area 2
  [133] = { "thim-map" },    -- THI area 3
  [141] = { "ttc-map" },     -- TTC area 1
  [151] = { "rr-map" },      -- RR area 1
  [201] = { "tsa-map" },     -- SA area 1
  [281] = { "btw-map" },     -- MCL area 1
  [311] = { "otr-map" },     -- RC area 1
  [211] = { "bits-map" },    -- BITS area 1
  [341] = { "bitsb-map" },   -- bitsb area 1
  [181] = { "vcutm-map" },   -- vcutm area 1
  [291] = { "totwc-map" },   -- totwc area 1
}

TEX_MAP_ARROW = get_texture_info('map-arrow')

-- minimap
TEX_HUD_STAR_GREYSCALE = get_texture_info('hud_star_greyscale')
TEX_HUD_BOX = get_texture_info('hud_box')
icon_minimap = {}
star_minimap = {}
box_minimap = {}
for i = 0, (MAX_PLAYERS - 1) do
  icon_minimap[i] = { tex = nil, prevX = 0, prevY = 0 } -- overriden with this character's head
end
for i = 1, 7 do
  star_minimap[i] = { tex = gTextures.star, prevX = 0, prevY = 0 }
end
for i = 1, 7 do
  box_minimap[i] = { tex = TEX_HUD_BOX, prevX = 0, prevY = 0 }
end
ex_minimap = {}
ex_minimap[1] = { tex = gTextures.coin, prevX = 0, prevY = 0 }
ex_minimap[2] = { tex = nil, prevX = 0, prevY = 0 } -- overriden with "S" in hud font

levelSize = 8192
defaultStarColor = { r = 255, g = 255, b = 92 } -- yellow

function render_on_minimap(m, mapData, isObj, objType)
  djui_hud_set_resolution(RESOLUTION_N64)
  local pos = {}
  local obj = m
  if not isObj then
    pos = { x = m.pos.x, y = m.pos.y + 80, z = m.pos.z } -- mario is 161 units tall
  else
    pos = { x = obj.oPosX, y = obj.oPosY, z = obj.oPosZ }
    if objType == "box" then        -- box's position is a bit higher
      pos.y = pos.y + 50
    elseif objType == "secret" then -- secrets have a misleading hitbox (although they aren't usually visible)
      pos.y = pos.y + 15
    elseif objType == "coin" then   -- these have their position centered at their bottom, so move up based on hitbox size
      pos.y = pos.y + obj.hitboxHeight / 2
    end
  end

  local screenWidth = djui_hud_get_screen_width()
  local screenHeight = djui_hud_get_screen_height()

  -- minimap render
  local renderSize = 80
  local x = screenWidth - 90
  local y = screenHeight - 120

  local tex = mapData.tex

  local level = gNetworkPlayers[0].currLevelNum
  local area = gNetworkPlayers[0].currAreaIndex
  if minimap_data[level * 10 + area] and minimap_data[level * 10 + area][2] then
    levelSize = minimap_data[level * 10 + area][2]
  end

  if pos.x > levelSize + 5 or pos.x < -levelSize - 5 or pos.z > levelSize + 5 or pos.z < -levelSize - 5 then -- adjust size if oob
    levelSize = levelSize * 2
  end
  local xScaled = (pos.x / (levelSize * 2) + 0.5)
  local yScaled = (pos.z / (levelSize * 2) + 0.5)

  local scale = 0.5

  local renderX = x + xScaled * renderSize - 8 * scale
  local renderY = y + yScaled * renderSize - 8 * scale

  if objType == "coin" then
    djui_hud_set_color(255, 0, 0, 255) -- red
  elseif objType == "secret" then
    djui_hud_set_color(255, 255, 255, 255)
    djui_hud_set_font(FONT_HUD)
    djui_hud_print_text("S", renderX, renderY, 0.5)
  elseif isObj and objType == nil then
    local r, g, b = 255, 255, 255
    if _G.OmmEnabled
        and obj_has_model_extended(obj, E_MODEL_STAR) ~= 1
        and obj_has_model_extended(obj, E_MODEL_TRANSPARENT_STAR) ~= 1
        and _G.OmmApi.omm_get_setting(gMarioStates[0], "color") == 1 then
      -- do colored radar
      local np = gNetworkPlayers[0]
      local starColor = omm_star_colors[np.currCourseNum]
      tex = TEX_HUD_STAR_GREYSCALE
      if starColor then
        r, g, b = starColor.r, starColor.g, starColor.b
      else
        r, g, b = 252, 240, 65 -- yellow (try to match original hud color)
      end
    elseif gGlobalSyncTable.ee then
      tex = TEX_HUD_STAR_GREYSCALE
      r, g, b = 255, 0, 0 -- red
    end
    djui_hud_set_color(r, g, b, 255)
  else
    djui_hud_set_color(255, 255, 255, 255)
  end


  if not isObj then
    render_player_head(m.playerIndex, renderX, renderY, scale, scale)

    local playercolor = network_get_player_text_color_string(m.playerIndex)
    local r, g, b = convert_color(playercolor)
    djui_hud_set_color(r, g, b, 155)
    djui_hud_set_rotation(m.faceAngle.y, 0.5, 0.5)
    djui_hud_render_texture(TEX_MAP_ARROW, renderX - 8 * scale, renderY - 8 * scale, scale, scale)
    djui_hud_set_rotation(0, 0, 0)
  elseif tex then
    djui_hud_render_texture_interpolated(tex, mapData.prevX, mapData.prevY, scale, scale, renderX, renderY, scale,
      scale)
  end
  mapData.prevX = renderX
  mapData.prevY = renderY
end

function render_minimap()
  djui_hud_set_resolution(RESOLUTION_N64)
  local screenWidth = djui_hud_get_screen_width()
  local screenHeight = djui_hud_get_screen_height()

  local renderSize = 80
  local x = screenWidth - 90
  local y = screenHeight - 120

  local level = gNetworkPlayers[0].currLevelNum
  local area = gNetworkPlayers[0].currAreaIndex
  local tex
  if minimap_data[level * 10 + area] then
    local data = minimap_data[level * 10 + area][1]
    if type(data) == "string" then
      tex = get_texture_info(data)
    else
      tex = data
    end
  end

  if tex then
    -- border
    djui_hud_set_color(50, 50, 50, 200)
    djui_hud_render_rect(x - 1, y - 1, renderSize + 2, renderSize + 2)

    local scale = renderSize / tex.width
    djui_hud_set_color(255, 255, 255, 200)
    djui_hud_render_texture(tex, x, y, scale, scale)
  else
    djui_hud_set_color(255, 255, 255, 100)
    djui_hud_render_rect(x, y, renderSize, renderSize)
  end
end

function render_player_minimap()
  djui_hud_set_resolution(RESOLUTION_N64)
  local screenWidth = djui_hud_get_screen_width()
  local screenHeight = djui_hud_get_screen_height()

  local renderSize = 80
  local x = screenWidth - 90
  local y = screenHeight - 120

  local pos = gMarioStates[0].pos

  local level = gNetworkPlayers[0].currLevelNum
  local area = gNetworkPlayers[0].currAreaIndex
  if minimap_data and minimap_data[level * 10 + area] and minimap_data[level * 10 + area][2] then
    levelSize = minimap_data[level * 10 + area][2]
  end

  if pos.x > levelSize + 5 or pos.x < -levelSize - 5 or pos.z > levelSize + 5 or pos.z < -levelSize - 5 then -- adjust size if oob
    levelSize = levelSize * 2
  end
  local xScaled = (pos.x / (levelSize * 2) + 0.5)
  local yScaled = (pos.z / (levelSize * 2) + 0.5)

  local renderX = x + xScaled * renderSize - 4.8
  local renderY = y + yScaled * renderSize - 4.8

  djui_hud_set_color(255, 255, 255, 255)
  render_player_head(0, renderX, renderY, 0.6, 0.6)

  local playercolor = network_get_player_text_color_string(0)
  local r, g, b = convert_color(playercolor)
  djui_hud_set_color(r, g, b, 155)
  djui_hud_set_rotation(gMarioStates[0].faceAngle.y, 0.5, 0.5)
  djui_hud_render_texture(TEX_MAP_ARROW, renderX - 8 * 0.6, renderY - 8 * 0.6, 0.6, 0.6)
  djui_hud_set_rotation(0, 0, 0)
end

-- star color table for OMM, based on course number
omm_star_colors = {
  [COURSE_BOB] = { r = 71, g = 192, b = 71 },
  [COURSE_WF] = { r = 190, g = 190, b = 190 },
  [COURSE_JRB] = { r = 237, g = 176, b = 204 },
  [COURSE_CCM] = { r = 30, g = 255, b = 255 },
  [COURSE_BBH] = { r = 189, g = 148, b = 203 },
  [COURSE_HMC] = { r = 127, g = 127, b = 127 },
  [COURSE_LLL] = { r = 255, g = 25, b = 25 },
  [COURSE_SSL] = { r = 192, g = 246, b = 74 },
  [COURSE_DDD] = { r = 28, g = 169, b = 240 },
  [COURSE_SL] = { r = 255, g = 255, b = 255 },
  [COURSE_WDW] = { r = 168, g = 189, b = 208 },
  [COURSE_TTM] = { r = 198, g = 161, b = 124 },
  [COURSE_THI] = { r = 255, g = 200, b = 64 },
  [COURSE_TTC] = { r = 250, g = 182, b = 146 },
  [COURSE_RR] = { r = 241, g = 127, b = 237 },
}

-- renders player head... with color!
local PART_ORDER = {
  SKIN,
  HAIR,
  CAP,
}

HEAD_HUD = get_texture_info("hud_head_recolor")
WING_HUD = get_texture_info("hud_wing")
CS_ACTIVE = _G.charSelectExists

local defaultIcons = {
  [gTextures.mario_head] = true,
  [gTextures.luigi_head] = true,
  [gTextures.toad_head] = true,
  [gTextures.waluigi_head] = true,
  [gTextures.wario_head] = true,
}

-- the actual head render function.
--- @param index integer
--- @param x integer
--- @param y integer
--- @param scaleX number
--- @param scaleY number
function render_player_head(index, x, y, scaleX, scaleY, noSpecial, alwaysCap, alpha_)
  local m = gMarioStates[index]
  local np = gNetworkPlayers[index]

  local alpha = alpha_ or 255
  if (not noSpecial) and (m.marioBodyState.modelState & MODEL_STATE_NOISE_ALPHA) ~= 0 then
      alpha = math.max(alpha - 155, 0) -- vanish effect
  end

  if charSelectExists then
      djui_hud_set_color(255, 255, 255, alpha)
      local TEX_CS_ICON = charSelect.character_get_life_icon(index)
      if TEX_CS_ICON and not defaultIcons[TEX_CS_ICON] then
          djui_hud_render_texture(TEX_CS_ICON, x, y, scaleX / (TEX_CS_ICON.width * 0.0625),
              scaleY / (TEX_CS_ICON.width * 0.0625))
          if (not noSpecial) and m.marioBodyState.capState == MARIO_HAS_WING_CAP_ON then
              djui_hud_render_texture(WING_HUD, x, y, scaleX, scaleY) -- wing
          end
          return
      elseif TEX_CS_ICON == nil then
          djui_hud_set_font(FONT_HUD)
          djui_hud_print_text("?", x, y, scaleX)
          if (not noSpecial) and m.marioBodyState.capState == MARIO_HAS_WING_CAP_ON then
              djui_hud_render_texture(WING_HUD, x, y, scaleX, scaleY) -- wing
          end
          return
      end
  end
  local isMetal = false
  local capless = false

  local tileY = m.character.type
  for i = 1, #PART_ORDER do
      local color = { r = 255, g = 255, b = 255 }
      if (not noSpecial) and (m.marioBodyState.modelState & MODEL_STATE_METAL) ~= 0 then -- metal
          color = network_player_get_override_palette_color(np, METAL)
          djui_hud_set_color(color.r, color.g, color.b, alpha)
          isMetal = true

          if (not (noSpecial or alwaysCap)) and m.marioBodyState.capState == MARIO_HAS_DEFAULT_CAP_OFF then
              capless = true
              djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, 7 * 16, tileY * 16, 16, 16) -- capless metal
          else
              djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, 5 * 16, tileY * 16, 16, 16)
          end
          break
      end

      local part = PART_ORDER[i]
      if (not (noSpecial or alwaysCap)) and part == CAP and m.marioBodyState.capState == MARIO_HAS_DEFAULT_CAP_OFF then -- capless check
          capless = true
          part = HAIR
      elseif tileY == 2 or tileY == 7 then
          if part == CAP and capless then return end
          tileY = 7 -- use alt toad
          if part == HAIR then -- toad doesn't use hair except when cap is off
              if (not (noSpecial or alwaysCap)) and m.marioBodyState.capState == MARIO_HAS_DEFAULT_CAP_OFF then
                  capless = true
                  part = HAIR
              else
                  part = GLOVES
              end
          end
      end
      color = network_player_get_override_palette_color(np, part)

      djui_hud_set_color(color.r, color.g, color.b, alpha)
      if capless then
          djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, 6 * 16, tileY * 16, 16, 16) -- render hair instead of cap
      else
          djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, (i - 1) * 16, tileY * 16, 16, 16)
      end
  end

  if not isMetal then
      djui_hud_set_color(255, 255, 255, alpha)
      --djui_hud_render_texture(HEAD_HUD, x, y, scaleX, scaleY)
      djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, (#PART_ORDER) * 16, tileY * 16, 16, 16)

      if not capless then
          djui_hud_render_texture_tile(HEAD_HUD, x, y, scaleX, scaleY, (#PART_ORDER + 1) * 16, tileY * 16, 16, 16) -- hat emblem
          if (not noSpecial) and m.marioBodyState.capState == MARIO_HAS_WING_CAP_ON then
              djui_hud_render_texture(WING_HUD, x, y, scaleX, scaleY)                                        -- wing
          end
      end
  elseif m.marioBodyState.capState == MARIO_HAS_WING_CAP_ON then
      djui_hud_set_color(109, 170, 173, alpha)                -- blueish green
      djui_hud_render_texture(WING_HUD, x, y, scaleX, scaleY) -- wing
  end
end
